;****************************************************************************
;*  CalcCameraMatrix(camera *c)
;*
;*  Description: Creates camera matrix
;*
;*  In  : EAX     - pointer to camera
;*
;*  Out : None
;****************************************************************************/
 CalcCameraMatrix_:
     push   ebx

     mov    ebx, eax

     mov    [u.vec_x], 3f800000h ;1.0
     mov    [u.vec_y], 0
     mov    [u.vec_z], 0

     mov    [v.vec_x], 0
     mov    [v.vec_y], 3f800000h ;1.0
     mov    [v.vec_z], 0

     mov    [n.vec_x], 0
     mov    [n.vec_y], 0
     mov    [n.vec_z], 3f800000h ;1.0

     fld    d [ebx.c_roll]
     fsincos
     fxch   st(1)
     fmul   d [Value_min_1]
     fxch   st(1)
     fstp   d [v.vec_y]
     fstp   d [v.vec_x]

     push   o n
     push   d [ebx.c_tgt_z]
     push   d [ebx.c_tgt_y]
     push   d [ebx.c_tgt_x]
     push   d [ebx.c_pos_z]
     push   d [ebx.c_pos_y]
     push   d [ebx.c_pos_x]
     call   VectorCreate_

     mov    eax, o n
     call   VectorNormalize_

     mov    eax, [n.vec_x]
     mov    [ebx.c_dir_x], eax
     mov    eax, [n.vec_y]
     mov    [ebx.c_dir_y], eax
     mov    eax, [n.vec_z]
     mov    [ebx.c_dir_z], eax

     push   d [n.vec_z]
     push   d [n.vec_y]
     push   d [n.vec_x]
     push   d [v.vec_z]
     push   d [v.vec_y]
     push   d [v.vec_x]
     call   VectorDot_

     ;  v.x -= (temp * n.x);  v.y -= (temp * n.y);  v.z -= (temp * n.z);
     ;                    st0  st1  st2  st3  st4  st5
                        ; temp
     fld    d [n.vec_x] ; nx   temp
     fld    d [n.vec_y] ; ny   nx   temp
     fld    d [n.vec_z] ; nz   ny   nx   temp
     fmul   st(0), st(3); okz  ny   nx   temp
     fxch   st(1)       ; ny   okz  nx   temp
     fmul   st(0), st(3); oky  okz  nx   temp
     fxch   st(2)       ; nx   okz  oky  temp
     fmulp  st(3), st(0); okz  oky  okx
     fld    d [v.vec_x] ; vx   okz  oky  okx
     fld    d [v.vec_y] ; vy   vx   okz  oky  okx
     fld    d [v.vec_z] ; vz   vy   vx   okz  oky  okx
     fsubrp st(3), st(0); vy   vx   ookz oky  okx
     fsubrp st(3), st(0); vx   ookz ooky okx
     fsubrp st(3), st(0); ookz ooky ookx
     fstp   d [v.vec_z]
     fstp   d [v.vec_y]
     fstp   d [v.vec_x]

     mov    eax, o v
     call   VectorNormalize_

     push   o u
     push   d [n.vec_z]
     push   d [n.vec_y]
     push   d [n.vec_x]
     push   d [v.vec_z]
     push   d [v.vec_y]
     push   d [v.vec_x]
     call   VectorCross_

     mov    eax, [u.vec_x]
     mov    [ebx.c_mtx], eax
     mov    eax, [u.vec_y]
     mov    [ebx.c_mtx + 1*4*4], eax
     mov    eax, [u.vec_z]
     mov    [ebx.c_mtx + 2*4*4], eax

     mov    eax, [v.vec_x]
     mov    [ebx.c_mtx +1*4], eax
     mov    eax, [v.vec_y]
     mov    [ebx.c_mtx + 1*4*4 +1*4], eax
     mov    eax, [v.vec_z]
     mov    [ebx.c_mtx + 2*4*4 +1*4], eax

     mov    eax, [n.vec_x]
     mov    [ebx.c_mtx +2*4], eax
     mov    eax, [n.vec_y]
     mov    [ebx.c_mtx + 1*4*4 +2*4], eax
     mov    eax, [n.vec_z]
     mov    [ebx.c_mtx + 2*4*4 +2*4], eax

     fld    d [ebx.c_pos_x]
     fmul   d [u.vec_x]
     fld    d [ebx.c_pos_y]
     fmul   d [u.vec_y]
     fld    d [ebx.c_pos_z]
     fmul   d [u.vec_z]
     fxch   st(2)
     faddp  st(1), st(0)
     faddp  st(1), st(0)
     fmul   d [Value_min_1]
     fstp   d [ebx.c_mtx + 3*4*4 + 0*4]

     fld    d [ebx.c_pos_x]
     fmul   d [v.vec_x]
     fld    d [ebx.c_pos_y]
     fmul   d [v.vec_y]
     fld    d [ebx.c_pos_z]
     fmul   d [v.vec_z]
     fxch   st(2)
     faddp  st(1), st(0)
     faddp  st(1), st(0)
     fmul   d [Value_min_1]
     fstp   d [ebx.c_mtx + 3*4*4 + 1*4]

     fld    d [ebx.c_pos_x]
     fmul   d [n.vec_x]
     fld    d [ebx.c_pos_y]
     fmul   d [n.vec_y]
     fld    d [ebx.c_pos_z]
     fmul   d [n.vec_z]
     fxch   st(2)
     faddp  st(1), st(0)
     faddp  st(1), st(0)
     fmul   d [Value_min_1]
     fstp   d [ebx.c_mtx + 3*4*4 + 2*4]

     pop    ebx
     ret
